/* Modified by Scott Hall shall@shall.dhs.org
   Original - dentar@dentar.com
   25/10/99 - made speech only say user defined message not whole line in log
   25/10/99 - added sleep(3) to for loop to stop excessive CPU usage (96%!!)
   26/10/99 - moved strcat(chmodcmd,"chmod 666 "); and 
              strcat(chmodcmd,ttyfilename); to after valid tty checks to stop
	      core dumps.
   26/10/99 - added -P for speech (whole line)
   26/10/99 - added -S for sleep time specification
*/

#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

int main(argc, argv, envp)
	int argc;
	char **argv[];
	char **envp[];
{
	time_t curtime=0;
	int stringlen=0;
	int sleepytime=3;
	int daemonize=0;
	int forky=0;
	int help=0;
	int wholeline=0;
	int speech=0;
	int debug=0;
	FILE *filehan=NULL;
	FILE *ttyout=NULL;
	FILE *logfile=NULL;
	FILE *speechfile=NULL;
	char *VERSTRING="0.41";
	char *logfilename=NULL;
	char *filename=NULL;
	char *ttyfilename=NULL;
	char *string=NULL;
	char *messagesay=NULL;  
	char *inputstring=NULL;
	char *chmodcmd=NULL;
	char argy=(char)0;

	inputstring=(char *)calloc(1,4096);

	chmodcmd=(char *)calloc(1,256);
	
	do
	{
		/* : means get after input parameter */ 
		argy = getopt(argc,(char **)argv,"h?Ddl:wt:f:s:S:p:PV");

		switch (argy)
		{
			case 'h':	help=1;
					break;
			case '?':	help=1;
					break;
			case 'D':	debug=1;
					break;
			case 'd':	daemonize=1;
					break;
			case 'l':	logfilename=(char *)calloc(1,1024);
					strcpy(logfilename,optarg);
					break;
			case 'w':	wholeline=1;
					break;
			case 't':	ttyfilename=(char *)calloc(1,1024);
					strcpy(ttyfilename,optarg);
					break;
			case 'f':	filename=(char *)calloc(1,1024);
					strcpy(filename,optarg);
					break;
			case 's':	string=(char *)calloc(1,1024);
					strcpy(string,optarg);
					stringlen=strlen(string);
					break;
			case 'S':	sleepytime=atoi(optarg);
					break;
			case 'p':	if (speech==0) speech=1;
					else
						printf ("ignoring -p\n");
					/* added to capture user defined message */
					messagesay=(char *)calloc(1,1024);
					strcpy(messagesay,optarg); 
					break;
			case 'P':
					if (speech==0)
						speech=2;
					else
						printf ("ignoring -P\n");
					break;
			case 'V':
					printf("\ntailbeep %s\n",VERSTRING);
					printf("by Tommy, dentar@dentar.com\n");
					printf("Modified Tony Gonzalez and Scott Hall\n");
					exit (0);
			
			default:	;
		}
		
	} while (argy != EOF);

	if (help==1)
	{
		printf("\nUsage: %s -f (filename) -t (tty) -s (string) [-l (filename) [-w]] [-P | -p \"(message)\"] [-d] [-D]\n",(char *)argv[0]);
		printf("\n");
		printf("  -l filename   append to log file.\n");
		printf("  -w            show whole line in log file. (-l required)\n");
		/* updated to include new -p format */
		printf("  -p \"message\"  speak \"message\" via /dev/speech (requires speechd)\n");
		printf("  -P            speak found lines via /dev/speech (requires speechd)\n");
		printf("* -f filename   filename is the file to watch.\n");
		printf("* -t /dev/ttyxx tty to send the beeps to.\n");
		printf("* -s string     string to watch for in filename.\n");
		printf("  -S integer    sleep for integer seconds between checks.\n");
		printf("  -d            daemonize.\n");
		printf("  -D            Debug mode.\n");
		printf("  -V            Version information\n");
		printf("(note: * = required)\n");
		printf("\n");
		exit(0);
	}

	if (daemonize==1)
	{
		forky=fork();

		if (forky>0) {
			printf ("Forked!\n");
			exit(0);
		}
		else if (forky<0)
		{
			fprintf (stderr,"tailbeep: fork failed.\n");
			fflush(stderr);
			exit(1);
		}
	}
	
	if ((filename==NULL) || (string==NULL) || (ttyfilename==NULL))
	{
		fprintf (stderr,"tailbeep: You MUST specify options -t, -f and -s with parameters!\n          (Parameters are also required for the optional flags -l & -p)\n");
		fflush(stderr);
		exit(1);
	}

	strcat(chmodcmd,"chmod 666 "); /* moved to after valid tty check */
	strcat(chmodcmd,ttyfilename);  /* moved to after valid tty check */

	if ((filehan=fopen(filename,"r"))==NULL)
	{
		fprintf (stderr,"tailbeep: Can't open %s!\n",filename);
		fflush(stderr);
		exit(2);
	}

	if ((fseek(filehan,0L,SEEK_END))!=0)
	{
		fprintf (stderr,"tailbeep: Can't seek to end of file %s!\n",filename);
		fflush(stderr);
		exit(2);
	}

	if ((ttyout=fopen(ttyfilename,"a+"))==NULL)
	{
		fprintf (stderr,"tailbeep: Can't open %s!\n",ttyfilename);
		fflush(stderr);
		exit(2);
	}

	if (speech==1)
	{
		if (debug==1) fprintf(stderr,"Opening speech...\n"); fflush(stderr);
		if((speechfile=fopen("/dev/speech","w"))==NULL)
		{
			fprintf(stderr,"tailbeep: Can't open /dev/speech!  Did you install speechd?");
			fflush(stderr);
			fclose(speechfile);
			speech=0;
		}

		if (debug==1) fprintf(stderr,"Done opening speech...\n"); fflush(stderr);
	}

	if (debug==1) fprintf(stderr,"Listening...\n");
	fflush(stderr);

	if (logfilename!=NULL)
	{
		if ((logfile=fopen(logfilename,"a+"))!=(FILE *)NULL)
		{
			fprintf(logfile,"Program started...\n");
			fflush(logfile);
		} else
		{
			fprintf (stderr,"tailbeep: Can't open log file %s\n",logfilename);
			exit (1);
		}
	} else {
		if (wholeline==1)
			fprintf (stderr,"tailbeep: Ignoring -w because -l not specified.\n");
	}
	
	for (;;)
	{
		memset(inputstring,0,4096);
		fgets(inputstring,1024,filehan);
		if (strstr(inputstring,string)!=NULL)
		{
			curtime=time(NULL);
			if (logfilename!=(char *)NULL)
			{
				if (wholeline==1)
				{
					fprintf(logfile,"%s -> %s",
					inputstring, ctime(&curtime));
				} else {
					fprintf(logfile,"Saw \"%s\" in %s at %s",
					string, filename, ctime(&curtime));
				}
				fflush(logfile);
			}

			if (debug==1) fprintf(stderr,"Saw \"%s\" in %s at %s",
					string, filename, ctime(&curtime));
			fflush(stderr);
			if (debug==1) fprintf(ttyout,"Saw \"%s\" in %s at %s",
					string, filename, ctime(&curtime));
			fprintf(ttyout,"%c",7);
			fflush(ttyout);

			if (speech==1)
			{
				/* changed fprintf(speechfile,"%s",inputstring); 
				to fprintf(speechfile,messagesay); 
				to stop whole log entry being read out now user
				defined "message" is said by speech */
				fprintf(speechfile,"%s\n",messagesay);
				fflush(speechfile);
			}
		}
	sleep(sleepytime); /* added to stop excessive CPU usage */
	}
}
